@@ -17,13 +17,12 @@ import sqlite3
 import itertools
 from pickle import dump, load
 import re
-from gettext import gettext as _
 from time import time, sleep
 from threading import Lock
 
 from eolie.helper_task import TaskHelper
 from eolie.sqlcursor import SqlCursor
-from eolie.define import EOLIE_DATA_PATH, ADBLOCK_JS
+from eolie.define import EOLIE_DATA_PATH
 from eolie.utils import remove_www
 from eolie.logger import Logger
 
@@ -199,33 +198,12 @@ class DatabaseAdblock:
         """
         if not Gio.NetworkMonitor.get_default().get_network_available():
             return
-        # Update adblock_js repo
-        git = GLib.find_program_in_path("git")
-        if git is None:
-            Logger.info(_("For stronger ad blocking, install git command"))
-        else:
-            if GLib.file_test(ADBLOCK_JS, GLib.FileTest.IS_DIR):
-                argv = [git,
-                        "-C",
-                        ADBLOCK_JS,
-                        "pull",
-                        "https://gitlab.gnome.org/gnumdk/eolie-adblock.git"]
-            else:
-                argv = [git,
-                        "clone",
-                        "https://gitlab.gnome.org/gnumdk/eolie-adblock.git",
-                        ADBLOCK_JS]
-            (pid, a1, a2, a3) = GLib.spawn_async(
-                argv,
-                flags=GLib.SpawnFlags.STDOUT_TO_DEV_NULL)
-            GLib.spawn_close_pid(pid)
-
         # DB version is last successful sync mtime
         try:
             version = load(open(EOLIE_DATA_PATH + "/adblock.bin", "rb"))
         except:
             version = 0
-        self.__cancellable.reset()
+        self.__cancellable = Gio.Cancellable.new()
         if self.__adblock_mtime - version > self.__UPDATE:
             # Update host rules
             uris = list(self.__URIS)
@@ -304,55 +282,59 @@ class DatabaseAdblock:
             @param netloc as str
             @return bool
         """
-        # We cache result for allowed uris
-        # because regex are quite slow in python
-        with SqlCursor(self) as sql:
-            result = sql.execute("SELECT allowed_uri\
-                                  FROM adblock_cache\
-                                  WHERE allowed_uri=?", (uri,))
-            v = result.fetchone()
-            if v is None:
-                # Search in main regexes
-                if self.__regex is None:
-                    request = "SELECT regex FROM adblock_re"
-                    result = sql.execute(request)
-                    rules = list(itertools.chain(*result))
-                    if rules:
-                        regexes = "|".join(regex for regex in rules)
-                        self.__regex = re.compile(regexes)
-                if self.__regex is not None:
-                    blocked_re = bool(self.__regex.search(uri))
-                else:
-                    blocked_re = False
-                # Find in domain regexes
-                request = "SELECT regex FROM adblock_re_domain\
-                           WHERE domain=?"
-                result = sql.execute(request, (netloc,))
-                rules = list(itertools.chain(*result))
-                if rules:
-                    regexes = "|".join(regex for regex in rules)
-                    blocked_re_domain = bool(re.search(regexes, uri))
-                else:
-                    blocked_re_domain = False
-                # If previous regexes blocked uri, check for an exception
-                if blocked_re_domain:
-                    request = "SELECT regex FROM adblock_re_domain_ex\
+        try:
+            # We cache result for allowed uris
+            # because regex are quite slow in python
+            with SqlCursor(self) as sql:
+                result = sql.execute("SELECT allowed_uri\
+                                      FROM adblock_cache\
+                                      WHERE allowed_uri=?", (uri,))
+                v = result.fetchone()
+                if v is None:
+                    # Search in main regexes
+                    if self.__regex is None:
+                        request = "SELECT regex FROM adblock_re"
+                        result = sql.execute(request)
+                        rules = list(itertools.chain(*result))
+                        if rules:
+                            regexes = "|".join(regex for regex in rules)
+                            self.__regex = re.compile(regexes)
+                    if self.__regex is not None:
+                        blocked_re = bool(self.__regex.search(uri))
+                    else:
+                        blocked_re = False
+                    # Find in domain regexes
+                    request = "SELECT regex FROM adblock_re_domain\
                                WHERE domain=?"
                     result = sql.execute(request, (netloc,))
                     rules = list(itertools.chain(*result))
                     if rules:
                         regexes = "|".join(regex for regex in rules)
-                        if bool(re.search(regexes, uri)):
-                            blocked_re_domain = False
-                if not blocked_re and not blocked_re_domain:
-                    sql.execute("INSERT INTO adblock_cache\
-                                (allowed_uri) VALUES (?)",
-                                (uri,))
-                    return False
+                        blocked_re_domain = bool(re.search(regexes, uri))
+                    else:
+                        blocked_re_domain = False
+                    # If previous regexes blocked uri, check for an exception
+                    if blocked_re_domain:
+                        request = "SELECT regex FROM adblock_re_domain_ex\
+                                   WHERE domain=?"
+                        result = sql.execute(request, (netloc,))
+                        rules = list(itertools.chain(*result))
+                        if rules:
+                            regexes = "|".join(regex for regex in rules)
+                            if bool(re.search(regexes, uri)):
+                                blocked_re_domain = False
+                    if not blocked_re and not blocked_re_domain:
+                        sql.execute("INSERT INTO adblock_cache\
+                                    (allowed_uri) VALUES (?)",
+                                    (uri,))
+                        return False
+                    else:
+                        return True
                 else:
-                    return True
-            else:
-                return False
+                    return False
+        except Exception as e:
+            Logger.error("DatabaseAdblock::is_uri_blocked(): %s", e)
+            return False
 
     def get_cursor(self):
         """
@@ -660,8 +642,18 @@ class DatabaseAdblock:
                              WHERE mtime!=?", (self.__adblock_mtime,))
                 sql.execute("DELETE FROM adblock_cache")
                 try:
-                    dump(self.__adblock_mtime,
-                         open(EOLIE_DATA_PATH + "/adblock.bin", "wb"))
+                    r1 = sql.execute("SELECT COUNT(*) from adblock")
+                    r2 = sql.execute("SELECT COUNT(*) from adblock_css")
+                    v1 = r1.fetchone()
+                    v2 = r2.fetchone()
+                    c1 = c2 = 0
+                    if v1 is not None:
+                        c1 = v1[0]
+                    if v2 is not None:
+                        c2 = v2[0]
+                    if c1 != 0 and c2 != 0:
+                        dump(self.__adblock_mtime,
+                             open(EOLIE_DATA_PATH + "/adblock.bin", "wb"))
                 except Exception as e:
                     Logger.error("DatabaseAdblock::__on_save_rules(): %s", e)
 
@@ -683,3 +675,4 @@ class DatabaseAdblock:
                                        callback=(self.__on_save_rules, uris))
         else:
             self.__on_save_rules(None, uris)
+            Logger.error("DatabaseAdblock::__on_load_uri_content(): %s", uri)
