@@ -1,4 +1,4 @@
-# Copyright (c) 2017 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2017-2018 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -16,11 +16,14 @@ import sqlite3
 import itertools
 from urllib.parse import urlparse
 from threading import Lock
+from re import sub
 
 from eolie.utils import noaccents, get_random_string
-from eolie.define import EOLIE_DATA_PATH
+from eolie.define import EOLIE_DATA_PATH, Type
 from eolie.localized import LocalizedCollation
 from eolie.sqlcursor import SqlCursor
+from eolie.logger import Logger
+from eolie.database_upgrade import DatabaseUpgrade
 
 
 class DatabaseBookmarks:
@@ -43,8 +46,7 @@ class DatabaseBookmarks:
                                                atime REAL NOT NULL,
                                                guid TEXT NOT NULL,
                                                mtime REAL NOT NULL,
-                                               position INT DEFAULT 0,
-                                               del INT DEFAULT 0
+                                               position INT DEFAULT 0
                                                )'''
     __create_tags = '''CREATE TABLE tags (id INTEGER PRIMARY KEY,
                                           title TEXT NOT NULL)'''
@@ -63,6 +65,7 @@ class DatabaseBookmarks:
         """
             Create database tables or manage update if needed
         """
+        upgrade = DatabaseUpgrade(Type.BOOKMARK)
         self.thread_lock = Lock()
         if not GLib.file_test(self.DB_PATH, GLib.FileTest.IS_REGULAR):
             try:
@@ -74,11 +77,13 @@ class DatabaseBookmarks:
                     sql.execute(self.__create_tags)
                     sql.execute(self.__create_bookmarks_tags)
                     sql.execute(self.__create_parents)
-                    sql.commit()
+                    sql.execute("PRAGMA user_version=%s" % upgrade.version)
             except Exception as e:
-                print("DatabaseBookmarks::__init__(): %s" % e)
+                Logger.error("DatabaseBookmarks::__init__(): %s", e)
+        else:
+            upgrade.upgrade(self)
 
-    def add(self, title, uri, guid, tags, atime=0, commit=True):
+    def add(self, title, uri, guid, tags, atime=0):
         """
             Add a new bookmark
             @param title as str
@@ -87,7 +92,6 @@ class DatabaseBookmarks:
             @param tags as [str]
             @param parent_guid as str
             @param ctime as int
-            @param commit as bool
             @return bookmark id as int
         """
         # Find an uniq guid
@@ -111,25 +115,9 @@ class DatabaseBookmarks:
                 sql.execute("INSERT INTO bookmarks_tags\
                              (bookmark_id, tag_id) VALUES (?, ?)",
                             (bookmarks_id, tag_id))
-            if commit:
-                sql.commit()
             return bookmarks_id
 
-    def delete(self, bookmark_id, delete=True, commit=True):
-        """
-            Mark bookmark as deleted
-            @param bookmark id as int
-            @param delete as bool
-            @param commit as bool
-        """
-        with SqlCursor(self) as sql:
-            sql.execute("UPDATE bookmarks\
-                         SET del=?\
-                         WHERE rowid=?", (delete, bookmark_id))
-            if commit:
-                sql.commit()
-
-    def remove(self, bookmark_id, commit=True):
+    def remove(self, bookmark_id):
         """
             Remove bookmark from db
             @param bookmark id as int
@@ -142,10 +130,8 @@ class DatabaseBookmarks:
                          WHERE bookmark_id=?", (bookmark_id,))
             sql.execute("DELETE FROM parents\
                          WHERE bookmark_id=?", (bookmark_id,))
-            if commit:
-                sql.commit()
 
-    def add_tag(self, tag, commit=False):
+    def add_tag(self, tag):
         """
             Add tag to db, return existing if exists
             @param tag as str
@@ -155,11 +141,9 @@ class DatabaseBookmarks:
             result = sql.execute("INSERT INTO tags\
                                   (title) VALUES (?)",
                                  (tag,))
-            if commit:
-                sql.commit()
             return result.lastrowid
 
-    def del_tag(self, tag, commit=False):
+    def del_tag(self, tag):
         """
             Add tag to db, return existing if exists
             @param tag as str
@@ -172,8 +156,6 @@ class DatabaseBookmarks:
                          WHERE rowid=?", (tag_id,))
             sql.execute("DELETE FROM bookmarks_tags\
                          WHERE tag_id=?", (tag_id,))
-            if commit:
-                sql.commit()
 
     def rename_tag(self, old, new):
         """
@@ -183,7 +165,6 @@ class DatabaseBookmarks:
         """
         with SqlCursor(self) as sql:
             sql.execute("UPDATE tags set title=? WHERE title=?", (new, old))
-            sql.commit()
 
     def get_tags(self, bookmark_id):
         """
@@ -230,8 +211,7 @@ class DatabaseBookmarks:
         with SqlCursor(self) as sql:
             result = sql.execute("SELECT rowid\
                                   FROM bookmarks\
-                                  WHERE uri=?\
-                                  AND del=0", (uri.rstrip('/'),))
+                                  WHERE uri=?", (uri.rstrip('/'),))
             v = result.fetchone()
             if v is not None:
                 return v[0]
@@ -262,19 +242,7 @@ class DatabaseBookmarks:
             result = sql.execute("SELECT rowid\
                                   FROM bookmarks\
                                   WHERE mtime > ?\
-                                  AND uri != guid\
-                                  AND del=0", (mtime,))
-            return list(itertools.chain(*result))
-
-    def get_deleted_ids(self):
-        """
-            Get ids that need to be synced related to mtime
-            @return [int]
-        """
-        with SqlCursor(self) as sql:
-            result = sql.execute("SELECT rowid\
-                                  FROM bookmarks\
-                                  WHERE del=1")
+                                  AND uri != guid", (mtime,))
             return list(itertools.chain(*result))
 
     def get_parent_guid(self, bookmark_id):
@@ -461,7 +429,6 @@ class DatabaseBookmarks:
                             WHERE bookmarks.rowid=bookmarks_tags.bookmark_id\
                             AND bookmarks_tags.tag_id=?\
                             AND bookmarks.guid != bookmarks.uri\
-                            AND bookmarks.del=0\
                             ORDER BY bookmarks.popularity DESC", (tag_id,))
             return list(result)
 
@@ -477,8 +444,7 @@ class DatabaseBookmarks:
                                    bookmarks.uri,\
                                    bookmarks.title\
                             FROM bookmarks\
-                            WHERE bookmarks.del=0\
-                            AND popularity!=0\
+                            WHERE popularity!=0\
                             AND bookmarks.guid != bookmarks.uri\
                             ORDER BY bookmarks.popularity DESC,\
                             bookmarks.atime DESC\
@@ -499,7 +465,6 @@ class DatabaseBookmarks:
                             WHERE NOT EXISTS (\
                                 SELECT bookmark_id FROM bookmarks_tags\
                                 WHERE bookmark_id=bookmarks.rowid)\
-                            AND bookmarks.del=0\
                             AND bookmarks.guid != bookmarks.uri\
                             ORDER BY bookmarks.popularity DESC")
             return list(result)
@@ -514,8 +479,7 @@ class DatabaseBookmarks:
                                   bookmarks.uri,\
                                   bookmarks.title\
                                   FROM bookmarks\
-                                  WHERE bookmarks.del=0\
-                                  AND bookmarks.guid != bookmarks.uri\
+                                  WHERE bookmarks.guid != bookmarks.uri\
                                   ORDER BY bookmarks.mtime DESC")
             return list(result)
 
@@ -563,35 +527,40 @@ class DatabaseBookmarks:
                 return v[0]
             return 5
 
-    def set_title(self, bookmark_id, title, commit=True):
+    def set_guid(self, bookmark_id, guid):
+        """
+            Set bookmark guid
+            @param bookmark_id as int
+            @param guid as str
+        """
+        with SqlCursor(self) as sql:
+            sql.execute("UPDATE bookmarks\
+                         SET guid=?\
+                         WHERE rowid=?", (bookmark_id, guid))
+
+    def set_title(self, bookmark_id, title):
         """
             Set bookmark title
             @param bookmark id as int
             @param title as str
-            @param commit as bool
         """
         with SqlCursor(self) as sql:
             sql.execute("UPDATE bookmarks\
                          SET title=?\
                          WHERE rowid=?", (title, bookmark_id,))
-            if commit:
-                sql.commit()
 
-    def set_uri(self, bookmark_id, uri, commit=True):
+    def set_uri(self, bookmark_id, uri):
         """
             Set bookmark uri
             @param bookmark id as int
             @param uri as str
-            @param commit as bool
         """
         with SqlCursor(self) as sql:
             sql.execute("UPDATE bookmarks\
                          SET uri=?\
                          WHERE rowid=?", (uri.rstrip('/'), bookmark_id,))
-            if commit:
-                sql.commit()
 
-    def set_popularity(self, bookmark_id, popularity, commit=True):
+    def set_popularity(self, bookmark_id, popularity):
         """
             Set bookmark popularity
             @param bookmark id as int
@@ -602,16 +571,13 @@ class DatabaseBookmarks:
             sql.execute("UPDATE bookmarks\
                          SET popularity=?\
                          WHERE rowid=?", (popularity, bookmark_id,))
-            if commit:
-                sql.commit()
 
-    def set_parent(self, bookmark_id, parent_guid, parent_name, commit=True):
+    def set_parent(self, bookmark_id, parent_guid, parent_name):
         """
             Set parent id for bookmark
             @param bookmark_id as int
             @param parent_guid as str
             @param parent_name as str
-            @param commit as bool
         """
         with SqlCursor(self) as sql:
             result = sql.execute("SELECT parent_guid\
@@ -628,8 +594,6 @@ class DatabaseBookmarks:
                              SET parent_guid=?, parent_name=?\
                              WHERE bookmark_id=?",
                             (parent_guid, parent_name, bookmark_id))
-            if commit:
-                sql.commit()
 
     def set_access_time(self, uri, atime):
         """
@@ -640,9 +604,8 @@ class DatabaseBookmarks:
         with SqlCursor(self) as sql:
             sql.execute("UPDATE bookmarks\
                          SET atime=? where uri=?", (atime, uri.rstrip('/')))
-            sql.commit()
 
-    def set_mtime(self, bookmark_id, mtime, commit=True):
+    def set_mtime(self, bookmark_id, mtime):
         """
             Set bookmark sync time
             @param bookmark id as int
@@ -652,10 +615,8 @@ class DatabaseBookmarks:
         with SqlCursor(self) as sql:
             sql.execute("UPDATE bookmarks\
                          SET mtime=? where rowid=?", (mtime, bookmark_id))
-            if commit:
-                sql.commit()
 
-    def set_position(self, bookmark_id, position, commit=True):
+    def set_position(self, bookmark_id, position):
         """
             Set bookmark position
             @param bookmark id as int
@@ -666,8 +627,6 @@ class DatabaseBookmarks:
             sql.execute("UPDATE bookmarks\
                          SET position=? where rowid=?", (position,
                                                          bookmark_id))
-            if commit:
-                sql.commit()
 
     def set_tag_title(self, tag_id, title):
         """
@@ -677,7 +636,6 @@ class DatabaseBookmarks:
         """
         with SqlCursor(self) as sql:
             sql.execute("UPDATE tags SET title=? WHERE id=?", (title, tag_id,))
-            sql.commit()
 
     def set_more_popular(self, uri):
         """
@@ -691,10 +649,9 @@ class DatabaseBookmarks:
             v = result.fetchone()
             if v is not None:
                 sql.execute("UPDATE bookmarks set popularity=?\
-                             WHERE uri=?", (v[0]+1, uri))
-                sql.commit()
+                             WHERE uri=?", (v[0] + 1, uri))
 
-    def add_tag_to(self, tag_id, bookmark_id, commit=True):
+    def add_tag_to(self, tag_id, bookmark_id):
         """
             Add tag to bookmark
             @param tag id as int
@@ -705,10 +662,8 @@ class DatabaseBookmarks:
             sql.execute("INSERT INTO bookmarks_tags\
                          (bookmark_id, tag_id) VALUES (?, ?)",
                         (bookmark_id, tag_id))
-            if commit:
-                sql.commit()
 
-    def del_tag_from(self, tag_id, bookmark_id, commit=True):
+    def del_tag_from(self, tag_id, bookmark_id):
         """
             Remove tag from bookmark
             @param tag id as int
@@ -719,8 +674,6 @@ class DatabaseBookmarks:
             sql.execute("DELETE from bookmarks_tags\
                          WHERE bookmark_id=? and tag_id=?",
                         (bookmark_id, tag_id))
-            if commit:
-                sql.commit()
 
     def clean_tags(self):
         """
@@ -732,9 +685,7 @@ class DatabaseBookmarks:
                             SELECT bookmarks_tags.rowid\
                             FROM bookmarks, bookmarks_tags\
                             WHERE tags.rowid = bookmarks_tags.tag_id\
-                            AND bookmarks.rowid = bookmarks_tags.bookmark_id\
-                            AND bookmarks.del!=1)")
-            sql.commit()
+                            AND bookmarks.rowid = bookmarks_tags.bookmark_id)")
 
     def reset_popularity(self, uri):
         """
@@ -744,7 +695,6 @@ class DatabaseBookmarks:
         with SqlCursor(self) as sql:
             sql.execute("UPDATE bookmarks SET popularity=0 WHERE uri=?",
                         (uri,))
-            sql.commit()
 
     def is_empty(self):
         """
@@ -764,7 +714,6 @@ class DatabaseBookmarks:
             @param path as str
         """
         try:
-            self.thread_lock.acquire()
             from bs4 import BeautifulSoup
             SqlCursor.add(self)
             f = Gio.File.new_for_path(path)
@@ -801,17 +750,13 @@ class DatabaseBookmarks:
                                 tags = [parent_name]
                             # Add bookmark
                             bookmark_id = self.add(title, uri, None,
-                                                   tags, 0, False)
+                                                   tags, 0)
                             # Set position
-                            self.set_position(bookmark_id, position, False)
+                            self.set_position(bookmark_id, position)
                             position += 1
-            with SqlCursor(self) as sql:
-                sql.commit()
             SqlCursor.remove(self)
         except Exception as e:
-            print("DatabaseBookmarks::import_html:", e)
-        finally:
-            self.thread_lock.release()
+            Logger.error("DatabaseBookmarks::import_html(): %s", e)
 
     def import_chromium(self, chrome):
         """
@@ -821,7 +766,6 @@ class DatabaseBookmarks:
             @param chrome as bool
         """
         try:
-            self.thread_lock.acquire()
             SqlCursor.add(self)
             import json
             homedir = GLib.get_home_dir()
@@ -829,10 +773,10 @@ class DatabaseBookmarks:
                 path = homedir + "/.config/chrome/Default/Bookmarks"
             else:
                 path = homedir + "/.config/chromium/Default/Bookmarks"
+            status = False
             f = Gio.File.new_for_path(path)
-            if not f.query_exists():
-                return
-            (status, content, tag) = f.load_contents(None)
+            if f.query_exists():
+                (status, content, tag) = f.load_contents(None)
             if status:
                 data = content.decode("utf-8")
                 j = json.loads(data)
@@ -849,7 +793,7 @@ class DatabaseBookmarks:
                             parents.append((child["name"], child["children"]))
                         elif child["type"] == "url":
                             bookmarks.append((child["name"],
-                                             child["url"]))
+                                              child["url"]))
                     position = 0
                     for bookmark in bookmarks:
                         tags = [parent_name]
@@ -862,45 +806,30 @@ class DatabaseBookmarks:
                         if rowid is None:
                             # Add bookmark
                             bookmark_id = self.add(title, uri, None,
-                                                   tags, 0, False)
+                                                   tags, 0)
                             # Set position
-                            self.set_position(bookmark_id, position, False)
+                            self.set_position(bookmark_id, position)
                             position += 1
-                with SqlCursor(self) as sql:
-                    sql.commit()
-                SqlCursor.remove(self)
+            SqlCursor.remove(self)
         except Exception as e:
-            print("DatabaseBookmarks::import_chromium:", e)
-        finally:
-            self.thread_lock.release()
+            Logger.error("DatabaseBookmarks::import_chromium(): %s", e)
 
-    def import_firefox(self):
+    def import_firefox(self, profile):
         """
             Mozilla Firefox importer
+            @param profile as str
         """
         try:
-            self.thread_lock.acquire()
             SqlCursor.add(self)
-            firefox_path = GLib.get_home_dir() + "/.mozilla/firefox/"
-            d = Gio.File.new_for_path(firefox_path)
-            infos = d.enumerate_children(
-                'standard::name,standard::type',
-                Gio.FileQueryInfoFlags.NOFOLLOW_SYMLINKS,
-                None)
-            sqlite_path = None
-            for info in infos:
-                if info.get_file_type() == Gio.FileType.DIRECTORY:
-                    f = Gio.File.new_for_path(firefox_path +
-                                              info.get_name() +
-                                              "/places.sqlite")
-                    if f.query_exists():
-                        sqlite_path = f.get_path()
-                        break
-            if sqlite_path is not None:
-                c = sqlite3.connect(sqlite_path, 600.0)
+            path = "%s/.mozilla/firefox/%s/places.sqlite" % \
+                (GLib.get_home_dir(),
+                 profile)
+            f = Gio.File.new_for_path(path)
+            if f.query_exists():
+                c = sqlite3.connect(path, 600.0)
                 # Add bookmarks
                 bookmarks = self.__get_firefox_bookmarks(c)
-                for (title, uri,  parent_name, bookmark_guid,
+                for (title, uri, parent_name, bookmark_guid,
                      parent_guid, position) in bookmarks:
                     tags = self.__get_tags_for_firefox_bookmark(c,
                                                                 bookmark_guid)
@@ -916,10 +845,10 @@ class DatabaseBookmarks:
                             tags = [parent_name]
                         # Bookmarks and folder
                         bookmark_id = self.add(title, uri, bookmark_guid,
-                                               tags, 0, False)
+                                               tags, 0)
                         self.set_parent(bookmark_id, parent_guid,
-                                        parent_name, False)
-                        self.set_position(bookmark_id, position, False)
+                                        parent_name)
+                        self.set_position(bookmark_id, position)
                 # Add folders, we need to get them
                 # as Firefox needs children order
                 parents = self.__get_firefox_parents(c)
@@ -934,17 +863,13 @@ class DatabaseBookmarks:
                     if rowid is None:
                         # Bookmarks and folder
                         bookmark_id = self.add(title, uri, bookmark_guid,
-                                               [], 0, False)
+                                               [], 0)
                         self.set_parent(bookmark_id, parent_guid,
-                                        parent_name, False)
-                        self.set_position(bookmark_id, position, False)
-            with SqlCursor(self) as sql:
-                sql.commit()
+                                        parent_name)
+                        self.set_position(bookmark_id, position)
             SqlCursor.remove(self)
         except Exception as e:
-            print("DatabaseBookmarks::import_firefox:", e)
-        finally:
-            self.thread_lock.release()
+            Logger.error("DatabaseBookmarks::import_firefox(): %s", e)
 
     def exists_guid(self, guid):
         """
@@ -964,34 +889,24 @@ class DatabaseBookmarks:
             @param limit as int
             @return [(id, title, uri, score)] as [(int, str, str, int)]
         """
-        words = search.split(" ")
-        # Remove empty items
-        words = [value.strip() for value in words]
-        words = list(filter(lambda x: x != '', words))
-        if not words:
-            return []
+        words = sub("[^\w]", " ", search.lower()).split()
         items = []
         with SqlCursor(self) as sql:
             filters = ()
             for word in words:
-                if not word:
-                    continue
                 filters += ("%" + word + "%", "%" + word + "%")
             filters += (limit,)
 
             # Search items matching all words
             request = "SELECT rowid, title, uri\
-                       FROM bookmarks WHERE\
-                       bookmarks.guid != bookmarks.uri AND "
+                       FROM bookmarks WHERE "
             words_copy = list(words)
             while words_copy:
                 word = words_copy.pop(0)
                 if word:
-                    request += " (title LIKE ? OR uri LIKE ?)"
-                    if words_copy:
-                        request += " AND "
-            request += "ORDER BY length(uri) ASC,\
-                        popularity DESC, mtime DESC LIMIT ?"
+                    request += " (title LIKE ? OR uri LIKE ?) AND"
+            request += " guid != uri ORDER BY mtime DESC,\
+                        popularity DESC LIMIT ?"
 
             result = sql.execute(request, filters)
             items += list(result)
@@ -999,41 +914,49 @@ class DatabaseBookmarks:
             # Search items matching any word
             request = "SELECT rowid, title, uri\
                        FROM bookmarks WHERE\
-                       bookmarks.guid != bookmarks.uri AND "
+                       guid != uri"
             words_copy = list(words)
+            if words:
+                request += " AND ("
             while words_copy:
                 word = words_copy.pop(0)
                 if word:
-                    request += " (title LIKE ? OR uri LIKE ?)"
+                    request += "title LIKE ? OR uri LIKE ?"
                     if words_copy:
                         request += " OR "
-            request += "ORDER BY length(uri) ASC,\
-                        popularity DESC, mtime DESC LIMIT ?"
+            if words:
+                request += ")"
+            request += " ORDER BY mtime DESC, popularity DESC LIMIT ?"
             result = sql.execute(request, filters)
             items += list(result)
+
         # Do some scoring calculation on items
         scored_items = []
         uris = []
         for item in items:
+            uri = item[2]
+            if uri in uris:
+                continue
             score = 0
+            title = item[1].lower()
+            parsed = urlparse(uri)
+            if not parsed.path:
+                score += 2
+            elif not parsed.query:
+                score += 2
             for word in words:
-                # Title match
-                if item[1].find(word) != -1:
-                    score += 1
+                lower_word = word.lower()
+                # If netloc match word, +1
+                if parsed.netloc.find(lower_word) != -1:
+                    score += len(lower_word) * 2
                 # URI match
-                elif item[2].find(word) != -1:
-                    score += 1
-                    parsed = urlparse(item[2])
-                    # If netloc match word, +1
-                    if parsed.netloc.find(word + "."):
-                        score += 1
-                    # If root +1
-                    if not parsed.path:
-                        score += 1
-            scored_item = (item[0], item[1], item[2], score)
-            if item[2] not in uris:
-                scored_items.append(scored_item)
-                uris.append(item[2])
+                elif uri.find(lower_word) != -1:
+                    score += len(lower_word)
+                # Title match
+                elif title.find(lower_word) != -1:
+                    score += len(lower_word) * 2
+            scored_items.append((item[0], item[1], item[2], score))
+            uris.append(uri)
         return scored_items
 
     def get_cursor(self):
@@ -1045,7 +968,8 @@ class DatabaseBookmarks:
             c.create_collation('LOCALIZED', LocalizedCollation())
             c.create_function("noaccents", 1, noaccents)
             return c
-        except:
+        except Exception as e:
+            Logger.error("DatabaseBookmarks::get_cursor(): %s", e)
             exit(-1)
 
 #######################
