@@ -132,7 +132,7 @@ class Window(Gtk.ApplicationWindow):
         """
         uri = webview.uri
         if uri is not None:
-            self.toolbar.title.set_uri(uri)
+            self.toolbar.title.update()
             accept_tls = App().websettings.get_accept_tls(uri)
             self.toolbar.end.show_tls_button(accept_tls)
         if webview.popups:
@@ -227,15 +227,6 @@ class Window(Gtk.ApplicationWindow):
         """
         return self.__window_state & Gdk.WindowState.FULLSCREEN
 
-    @property
-    def modifiers(self):
-        """
-            Get modifier
-            return int
-        """
-        # https://bugs.webkit.org/show_bug.cgi?id=181041
-        return self.__modifiers
-
     @property
     def size(self):
         """
@@ -379,8 +370,6 @@ class Window(Gtk.ApplicationWindow):
             @param window as Window
             @param event as Gdk.EventKey
         """
-        if event.keyval in [Gdk.KEY_Shift_L, Gdk.KEY_Control_L]:
-            self.__modifiers = event.keyval
         if event.state & Gdk.ModifierType.CONTROL_MASK and\
                 event.keyval == Gdk.KEY_Tab:
             if not self.container.in_expose:
@@ -392,7 +381,6 @@ class Window(Gtk.ApplicationWindow):
             @param window as Window
             @param event as Gdk.EventKey
         """
-        self.__modifiers = 0
         if event.keyval == Gdk.KEY_Control_L:
             self.__container.ctrl_released()
         elif event.keyval == Gdk.KEY_Escape:
@@ -431,6 +419,11 @@ class Window(Gtk.ApplicationWindow):
             self.__container.try_close_view(self.container.current)
         elif string == "reload":
             self.container.current.webview.reload()
+        elif string == "settings":
+            # Rework all this code to use actions like in Lollypop
+            from eolie.settings import SettingsDialog
+            dialog = SettingsDialog(self)
+            dialog.show()
         elif string == "home":
             self.container.current.webview.load_uri(App().start_page)
         elif string == "source":
